import { coordCanvasToWebGL } from "./util.js";

/**
 * 顶点着色器代码
 */
const vertexShaderSource = `#version 300 es
in vec4 a_position;
out vec4 cur_position;

void main() {
    // float curPointSize = 60.0;
    // gl_PointSize = curPointSize;
    gl_Position = a_position;
    cur_position = a_position;
}
`;

/**
 * 片元着色器代码
 */
const fragmentShaderSource = `#version 300 es
precision mediump float;
in vec4 cur_position;
out vec4 color;

uniform vec3 iResolution;

// test data
// uniform int PointCount;
const int PointCount = 30;
uniform vec3 Points[30];

uniform float HEAT_MAX;
const float PointRadius = .75;

vec4 gradient(float w, vec2 uv) {
    w = pow(clamp(w, 0., 1.) * 3.14159 * .5, .9);
    vec3 c = vec3(sin(w), sin(w * 2.), cos(w)) * 1.1;
    float a = clamp(w, 0.5, 1.);
    return mix(vec4(0.25, 1.0, 0.0, 0.5), vec4(c, a), w*0.2);
}

void mainImage(out vec4 fragColor, in vec2 fragCoord) {
     vec2 uv = (2. * fragCoord - iResolution.xy) / min(iResolution.x, iResolution.y);

    float d = 0.;
    for (int i = 0; i < PointCount; i++) {
        vec3 v = Points[i];
        float intensity = v.z / HEAT_MAX;
        float dis = length(uv - v.xy);
        float pd = (1.0 - dis / PointRadius) * intensity;
        d += pow(max(0., pd), 2.);
    }

    fragColor = gradient(d, uv);
}

void main() {
    vec4 col = vec4(1.0, 0.0, 0.0, 1.0);
    mainImage(col, gl_FragCoord.xy);

    color = col;
}
`;


/**
 * webgl2绘制上下文
 */
var gl;

/**
 * 获取着色器源码-fn
 * @param {string} 文件名称
 * @returns {Promise<string>} 着色器源码
 */
function getShaderSource(filename) {
    return fetch(filename).then((res) => res.text());
}

/**
 * 创建着色器-fn
 * @param {GLenum} 着色器类型
 * @param {string} 着色器源码
 * @returns {WebGLShader} shader着色器
 */
function createShaderFn(shaderType, shaderSource) {
    // 创建着色器
    const shader = gl.createShader(shaderType);
    // 着色器添加源码
    gl.shaderSource(shader, shaderSource);
    // 编译着色器源码
    gl.compileShader(shader);
    if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
        const shaderTypeText = gl.VERTEX_SHADER === shaderType ? 'vertexShader' : 'fragmentShader';
        console.error(`Error compiling ${shaderTypeText}:`, gl.getShaderInfoLog(shader));  // 输出编译错误信息
    }
    return shader;
}

/**
 * 创建shader程序-fn
 * @param {WebGLShader} 顶点着色器
 * @param {WebGLShader} 片元着色器
 * @returns {WebGLProgram} shader程序
 */
function createProgramFn(vertexShader, fragmentShader) {
    // 创建shader程序
    const shaderProgram = gl.createProgram();
    // shader程序添加着色器
    gl.attachShader(shaderProgram, vertexShader);
    gl.attachShader(shaderProgram, fragmentShader);

    // 链接shader程序
    gl.linkProgram(shaderProgram);

    // 使用shader程序
    gl.useProgram(shaderProgram);
    return shaderProgram;
}

/**
 * 
 * @param {*} pCanvasInfo 主画布的信息(parent canvas宽度、高度等信息)
 * @param {*} tempPointList 测试的温度测点
 * @returns 
 */
function main(pCanvasInfo = {}, tempPointList  = []) {

    // 获取顶点着色器和片元着色器的GLSL代码
    // const vertexShaderSource = await getShaderSource('./vertShader.vs');
    // const fragmentShaderSource = await getShaderSource('./fragShader.fs');

    const canvasDom = document.createElement('canvas');
    canvasDom.width = pCanvasInfo.width || 1200;
    canvasDom.height = pCanvasInfo.height || 804;
    gl = canvasDom.getContext('webgl2');

    // 创建着色器(顶点、片元着色器)
    const vertexShader = createShaderFn(gl.VERTEX_SHADER, vertexShaderSource);
    const fragmentShader = createShaderFn(gl.FRAGMENT_SHADER, fragmentShaderSource);

    // 创建shader程序
    const shaderProgram = createProgramFn(vertexShader, fragmentShader);

    const canvasNdcList = [
        -1.0, 1.0,
        1.0, 1.0,
        -1.0, -1.0,
        1.0, -1.0,
    ];
    const triangleVertices = new Float32Array(canvasNdcList);
    // 创建缓冲区
    const vertexBuffer = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, triangleVertices, gl.STATIC_DRAW);

    // 获取a_position属性的位置
    var a_Position = gl.getAttribLocation(shaderProgram, 'a_position');
    // 将缓冲区的数据分配给a_Position变量
    gl.vertexAttribPointer(a_Position, 2, gl.FLOAT, false, 0, 0);
    // 开启a_Position变量
    gl.enableVertexAttribArray(a_Position);

    const iResolutionAddr = gl.getUniformLocation(shaderProgram, "iResolution");
    //给iResolutionAddr 字段设置数据 ，是canvas的像素宽高
    gl.uniform3f(iResolutionAddr, gl.canvas.width, gl.canvas.height, 1.0);

    const points = [];
    for (const item of tempPointList) {
        const point = coordCanvasToWebGL(item.x, item.y, gl.canvas.width, gl.canvas.height);
        point[2] = item.value;
        points.push(point);
    }


    // 获取 uniform 位置
    const uniformAddr = gl.getUniformLocation(shaderProgram, "Points");
    gl.uniform3fv(uniformAddr, new Float32Array(points.flat()));

    const HEATMAXAddr = gl.getUniformLocation(shaderProgram, "HEAT_MAX");
    gl.uniform1f(HEATMAXAddr, 35.5);

    // gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
    // gl.clearColor(0.0, 1.0, 0.0, 0.75); // 设置canvas的清除颜色
    // gl.clear(gl.COLOR_BUFFER_BIT); // 清除canvas
    // 执行绘制
    gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);

    return  gl.canvas;
}

function generateHeatmapCanvas(pCanvasInfo = {}, tempPointList = []) {
    const canvas = main(pCanvasInfo, tempPointList);
    
    return canvas;
    
}

async function getHeatmapImage(pCanvasInfo = {}, tempPointList = []) {
    const canvas = main(pCanvasInfo, tempPointList);
    
    return new Promise((resolve) => {
      const img = new Image();

      img.onload = () => {
        resolve(img);
      };

      img.src = canvas.toDataURL('image/png', 1.0);
    });
  }


export {
    generateHeatmapCanvas,
    getHeatmapImage,
}