/**
 * 齐次坐标(四维齐次坐标) 统一处理平移、旋转、缩放等变换
 * ` (x,y,z,w) `
 * - z 值代表深度测试, 来确定物体的前后关系
 * - w 值通常用来表示 位置 还是 方向; `(x, y, z, 1.0) 表示一个位置`  `(x, y, z, 0.0) 表示一个方向`
 */

/**
 * 坐标系转换-Canvas转WebGL  (Canvas TO  WebGL)返回的数值的精度很关键！！！
 * @param {number} 画布上x轴的点位坐标
 * @param {number} 画布上y轴的点位坐标
 * @param {number} 画布宽度
 * @param {number} 画布高度
 * @returns {number[]} ndc坐标(三维, 浮点数， 最多保留4位小数)
 * - Canvas 二维坐标系
 *      (像素级别)默认的坐标系的原点 (0, 0) 位于画布的左上角。坐标系中的 x 值沿水平方向增加，y 值沿垂直方向增加，即 x 向右增大，y 向下增大
 * - WebGL 三维坐标系(z轴默认填充0.0， z轴的分量代表深度测试, 来确定物体的前后关系)
 *      原点是 (0, 0, 0)，位于视口的中心。WebGL 使用的是一个标准化坐标系，范围通常是 [-1, 1]，这意味着 x 和 y 的值在 NDC 中会被标准化到这个范围内
 */
export function  coordCanvasToWebGL(x_canvas, y_canvas, canvasWidth, canvasHeight) {
    const x_ndc = +((x_canvas  / (canvasWidth / 2)) - 1).toFixed(6);
    const y_ndc = +(1 - (y_canvas  / (canvasHeight / 2))).toFixed(6);
    return [x_ndc, y_ndc, 0.0];
}

/**
 * 坐标系转换-Canvas转WebGL  (WebGL TO Canvas)
 * @param {number} 画布上x轴的点位坐标
 * @param {number} 画布上y轴的点位坐标
 * @param {number} 画布宽度
 * @param {number} 画布高度
 * @returns {number[]} 画布坐标(二维, 整数)
 * - Canvas 二维坐标系
 *      (像素级别)默认的坐标系的原点 (0, 0) 位于画布的左上角。坐标系中的 x 值沿水平方向增加，y 值沿垂直方向增加，即 x 向右增大，y 向下增大
 * - WebGL 三维坐标系(z轴默认填充0.0， z轴的分量代表深度测试, 来确定物体的前后关系)
 *      原点是 (0, 0)，位于视口的中心。WebGL 使用的是一个标准化坐标系，范围通常是 [-1, 1]，这意味着 x 和 y 的值在 NDC 中会被标准化到这个范围内
 */
export function  coordWebGLToCanvas(x_ndc, y_ndc, canvasWidth, canvasHeight) {
    const x_canvas = Math.ceil((x_ndc + 1) * (canvasWidth / 2));
    const y_canvas = Math.ceil((1 - y_ndc) * (canvasHeight / 2));
    return [x_canvas, y_canvas];
}

/**
 * 加载图片资源
 * @param {*} url 
 * @returns 
 */
export function loadImageUrl(url) {
    return new Promise((resolve, reject) => {
      const image = new Image();
      image.src = url;
      image.onload = () => {
        resolve({ url, image });
      };
  
      image.onerror = reject;
    });
  }


/**
 * 根据多边形的点构建矩形-用于计算热力图形状的顶点信息
 */
export function genHeatmapAreaByPoints(areaPoints = []) {
    let left = areaPoints[0][0];
    let right = areaPoints[0][0];
    let top = areaPoints[0][1];
    let bottom = areaPoints[0][1];
    for (const [x, y] of areaPoints) {
      if (x < left) {
        left = x; // 更新最左点
      }
      if (x > right) {
        right = x; // 更新最右点
      }
      if (y < top) {
        top = y; // 更新最上点
      }
      if (y > bottom) {
        bottom = y; // 更新最下点
      }
    }
    const width = right - left;
    const height = bottom - top;
  
    return {
      /**
       * 左上角
       */
      leftTop: [left, top],
      rightTop: [right, top],
      rightBottom: [right, bottom],
      /**
       * 左下角
       */
      leftBottom: [left, bottom],
      width,
      height,
    };
  }